package server.ai;

import java.util.function.Predicate;

import common.ai.AIRangedAttack;
import common.ai.EntityAIAttackOnCollide;
import common.ai.EntityAIAvoidEntity;
import common.ai.EntityAIHurtByTarget;
import common.ai.EntityAILookAtTalkingPlayer;
import common.ai.EntityAINagPlayer;
import common.ai.EntityAINearestAttackableTarget;
import common.ai.EntityAINpcInteract;
import common.ai.EntityAINpcMate;
import common.ai.EntityAIOpenDoor;
import common.ai.EntityAIPlay;
import common.ai.EntityAISwimming;
import common.ai.EntityAIWander;
import common.ai.EntityAIWatchClosest;
import common.ai.EntityAIWatchClosest2;
import common.entity.npc.EntityNPC;
import common.entity.types.EntityLiving;
import common.item.ItemStack;
import common.pathfinding.PathNavigateGround;

public class EntityNPCNode extends EntityNode {
    protected final EntityAIAttackOnCollide aiMelee;
    protected final AIRangedAttack aiRanged;
    private boolean fleeing;
	
	public EntityNPCNode(EntityNPC entity) {
		super(entity);
		this.aiMelee = new EntityAIAttackOnCollide(entity, EntityLiving.class, 1.0D, true, true) {
	    	public boolean shouldExecute()
	        {
	        	return !EntityNPCNode.this.fleeing && super.shouldExecute();
	        }
	    	
	        public boolean continueExecuting()
	        {
	        	return !EntityNPCNode.this.fleeing && super.shouldExecute();
	        }
	    };
	    this.aiRanged = new AIRangedAttack(entity, 1.0D, entity.getAttackSpeed(), entity.getAttackSpeed() * 3, 15.0F) {
//	    	public void startExecuting()
//	        {
//	            super.startExecuting();
//	            EntityNPC.this.setUsingItem(true);
//	        }
//	    	
//	    	public void resetTask()
//	        {
//	            super.resetTask();
//	            EntityNPC.this.setUsingItem(false);
//	        }
	    	
	        public boolean shouldExecute()
	        {
	        	return !EntityNPCNode.this.fleeing && super.shouldExecute();
	        }
	    	
	        public boolean continueExecuting()
	        {
	        	return !EntityNPCNode.this.fleeing && super.shouldExecute();
	        }
	    };
	}
	
	public EntityNPC getEntity() {
		return (EntityNPC)this.entity;
	}
	
	protected void registerTasks() {
        if(this.getNavigator() instanceof PathNavigateGround) {
	        ((PathNavigateGround)this.getNavigator()).setBreakDoors(true);
	        ((PathNavigateGround)this.getNavigator()).setAvoidsWater(true);
        }
        this.tasks.addTask(0, new EntityAISwimming(this.getEntity()));
//        this.tasks.addTask(1, new EntityAIAttackOnCollide(this, EntityNPC.class, 0.6D, true, true));
//        this.tasks.addTask(1, new EntityAIAttackOnCollide(this, EntityLivingBase.class, 0.6D, true, true));
        this.tasks.addTask(1, new EntityAINpcMate(this.getEntity()));
        this.tasks.addTask(2, new EntityAIAvoidEntity(this.getEntity(), EntityLiving.class, new Predicate<EntityLiving>() {
			public boolean test(EntityLiving entity) {
				return entity != EntityNPCNode.this.entity && EntityNPC.this.shouldFlee(entity);
			}
		}, 8.0F, 1.1D, 1.1D) {
        	{
        		this.setMutexBits(0);
        	}
        	
        	public void startExecuting() {
                EntityNPCNode.this.setAttackTarget(null);
                EntityNPCNode.this.fleeing = true;
            }
        	
        	public boolean continueExecuting() {
                return false;
            }

            public void updateTask()
            {
            }
        	
//        	public void resetTask() {
//                super.resetTask();
//                EntityNPC.this.isFleeing = false;
//            }
        });
        this.tasks.addTask(3, new EntityAIAvoidEntity(this, EntityLiving.class, new Predicate<EntityLiving>() {
			public boolean test(EntityLiving entity) {
				return entity != EntityNPC.this && EntityNPC.this.shouldFlee(entity);
			}
		}, 8.0F, 1.1D, 1.1D) {
        	public void startExecuting() {
        		super.startExecuting();
                EntityNPC.this.setAttackTarget(null);
                EntityNPC.this.fleeing = true;
            }
        	
        	public void resetTask() {
                super.resetTask();
                EntityNPC.this.fleeing = false;
            }
        });
//        this.tasks.addTask(2, new EntityAITempt(this, 1.0D, Items.golden_apple, false));
        this.tasks.addTask(4, new EntityAIOpenDoor(this, true));
        this.tasks.addTask(5, new EntityAINagPlayer(this));
        this.tasks.addTask(5, new EntityAILookAtTalkingPlayer(this));
        this.tasks.addTask(6, new EntityAIWatchClosest2(this, null, 3.0F, 1.0F) {
        	private int sneakTime;
        	
        	public void updateTask()
            {
                super.updateTask();
                boolean flag = this.closestEntity.isPlayer() && this.closestEntity.isSneaking();
                if(this.sneakTime > 0) {
                	if(--this.sneakTime == 0) {
                		EntityNPC.this.setSneaking(false);
                	}
                }
                else if(EntityNPC.this.getAttackTarget() == null && EntityNPC.this.rand.chance(flag ? 5 : 200)) {
                	EntityNPC.this.setSneaking(true);
                	this.sneakTime = EntityNPC.this.rand.range(60, flag ? 160 : 120);
                }
                else if(EntityNPC.this.getAttackTarget() != null) {
                	EntityNPC.this.setSneaking(false);
                	this.sneakTime = 0;
                }
            }
        	
        	public void resetTask()
            {
                super.resetTask();
                EntityNPC.this.setSneaking(false);
            }
        });
        this.tasks.addTask(7, new EntityAINpcInteract(this));
        this.tasks.addTask(8, new EntityAIWander(this, 1.0D));
        this.tasks.addTask(8, new EntityAIPlay(this, 1.1D));
        this.tasks.addTask(9, new EntityAIWatchClosest(this, EntityLiving.class, 8.0F));
        this.targets.addTask(1, new EntityAIHurtByTarget(this, false, /* EntityNPC.class, */ EntityLiving.class) {
            protected boolean isSuitableTarget(EntityLiving entity) {
            	if(entity != null && entity != EntityNPC.this && EntityNPC.this.shouldFlee(entity)) {
            		EntityNPC.this.setAttackTarget(null);
                    EntityNPC.this.fleeing = true;
            		return false;
            	}
				return entity != null && entity != EntityNPC.this && (!(entity.isPlayer()) || EntityNPC.this.rand.chance(entity.getAttackedBy() == EntityNPC.this ? 2 : 4))
						&& EntityNPC.this.canCounter(entity) && super.isSuitableTarget(entity);
            }
        });
        this.targets.addTask(2, new EntityAINearestAttackableTarget(this, EntityLiving.class, 10, true, false, new Predicate<EntityLiving>() {
			public boolean test(EntityLiving entity) {
            	if(entity != null && entity != EntityNPC.this && EntityNPC.this.shouldFlee(entity)) {
            		EntityNPC.this.setAttackTarget(null);
                    EntityNPC.this.fleeing = true;
            		return false;
            	}
				return entity != null && entity != EntityNPC.this && !EntityNPC.this.fleeing && EntityNPC.this.canAmbush(entity) && EntityNPC.this.canAttack(entity);
			}
		}));
//        this.setCanPickUpLoot(true);
        
        if (worldIn != null && !worldIn.client)
        {
            this.setCombatTask();
        }
	}
    
	public void updateLeashedState() {
		if(!this.player)
			super.updateLeashedState();
	}
	
    public void sendDeathMessage() {
		this.sendDeathMessage(this.player, true);
	}
    
    public int getMaxFallHeight()
    {
        return this.player ? 3 : super.getMaxFallHeight();
    }
    
    public void setCombatTask()
    {
    	if(!this.player) {
            this.tasks.removeTask(this.aiMelee);
            this.tasks.removeTask(this.aiRanged);
            ItemStack itemstack = this.getEntity().getHeldItem();

            if (this.getEntity().isRangedWeapon(itemstack))
            {
                this.tasks.addTask(3, this.aiRanged);
            }
            else
            {
                this.tasks.addTask(3, this.aiMelee);
            }
    	}
    }

	public boolean isFleeing() {
		return this.fleeing;
	}
}
